/*
 * Arduino code for controlling a DC motor's speed and direction with an ESP8266 using a browser on a device in local network.
 * 
 * Made by KushagraK7: https://www.instructables.com/member/KushagraK7/
 *
 * Modified to suit by Drewzero1: https://gardenrails.org/forum/viewtopic.php?t=14727
 * 
 * The ESP8266 provides a wireless network for the controlling device (e.g. phone, computer, tablet) to connect. NOTE: internet access will be unavailable while connecting to this network! Some devices will automatically leave the provided network in favor of a known network where internet access is available, and must be prevented from doing so to maintain control.
 * 
 * Connect to network "LocoControl" and point your browser to 172.0.0.1
 * 
 */

#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>


#include "PageIndex.h"; //--> Include the contents of the User Interface Web page, stored in the same folder as the .ino file

#define LEDonBoard 12  //--> Defining an On Board LED, used for indicators when the process of connecting to a wifi router

#define OutA 0
#define OutB 2

#define YES 1
#define NO 0

// ---- Set your locomotive name here ---
const char* loco_name = "LocoControl";

//----------------------------------------SSID and Password of your WiFi router
const char* ap_ssid = loco_name;
const char* ap_password = "";
uint8_t max_connections=8; // maximum connections allowed to the AP
IPAddress APIP(172, 0, 0, 1);
//DNSServer dnsServer; ESP8266WebServer webServer(80); // for DNS spoofing? Didn't work, try again
//const byte DNS_PORT = 53;

//----------------------------------------

int pos;
int p = 0;
int m = 1;
int sp;
float spe = 0;

int acc = YES;//If the value is 'YES', the motor will accelerate, if it is 'NO', it will not.

String POS;
String Dir;

// AP keep track of number of connections, for serial logging
int current_stations=0, new_stations=0;


ESP8266WebServer server(80);  //--> Server on port 80

//----------------------------------------This routine is executed when you open NodeMCU ESP8266 IP Address in browser
void handleRoot() {
 String s = MAIN_page; //Read HTML contents
 server.send(200, "text/html", s); //Send web page
}
//----------------------------------------

//----------------------------------------Procedure for handling servo control
void setSpeed()
{
  POS = server.arg("POS");
  
  p = POS.toInt(); 
  
  Serial.print("Position:  ");
  Serial.print(p);
  Serial.print("  Direction:  ");
  Serial.println(m);
  server.send(200, "text/plane","");

  
}

void setDir()
{
  Dir = server.arg("Dir");

  if(Dir.length()  != 0) m = Dir.toInt();

  /*
  Serial.print("Position:  ");
  Serial.print(p);
  Serial.print("  Direction:  ");
  Serial.println(m);
  server.send(200, "text/plane","");
  */
}

void run(int s)
{
  if(s > 0 && s <= 255)
  {
    digitalWrite(OutB, LOW);
    analogWrite(OutA, s);
  }

  if(s < 0 && s >= -255)
  {
    digitalWrite(OutA, LOW);
    analogWrite(OutB, -s);
  }

  else
  {
    digitalWrite(OutB, LOW);
    digitalWrite(OutA, LOW);
  }

}
//----------------------------------------
//----------------------------------------Setup----------------------------------------
void setup() {
  Serial.begin(115200);
  delay(1000);
  Serial.println();

  pinMode(OutA, OUTPUT);
  pinMode(OutB, OUTPUT);
  

  // SoftAP settings for AP Mode
  //Setting the AP Mode with SSID, Password, and Max Connection Limit
  if(WiFi.softAP(ap_ssid,ap_password,1,false,max_connections)==true)
  {
    Serial.print("Access Point is Created with SSID: ");
    Serial.println(ap_ssid);
    Serial.print("Max Connections Allowed: ");
    Serial.println(max_connections);
    // attempt to set IP address
    WiFi.softAPConfig(APIP, APIP, IPAddress(255, 255, 255, 0));
    Serial.print("Access Point IP: ");
    Serial.println(WiFi.softAPIP());
  }
  else
  {
    Serial.println("Unable to Create Access Point");
  }
  //  --------

  // ----------- DNS Spoofing (not working)
  //dnsServer.start(DNS_PORT, "*", APIP); // DNS spoofing (Only for HTTP)

  //----------------------------------------Initialize Webserver
  server.on("/",handleRoot);  //--> Routine to handle at root location. This is to display web page.
  server.on("/setPOS",setSpeed); //--> Sets servo position from Web request
  server.on("/setDir",setDir);
  server.begin();  
  Serial.println("HTTP server started");
}
//------------------------------------------------------------------------------------
//----------------------------------------Loop----------------------------------------
void loop() {
 server.handleClient();

 sp = map(p, 0, 100, 0, 255);

 if(spe < sp*m) spe+= 0.1;
 if(spe > sp*m) spe-= 0.1;

 if(acc == 1) run(spe);
 else run(sp*m);
}
//------------------------------------------------------------------------------------
